from ._violin import ViolinsValidator
from ._table import TablesValidator
from ._surface import SurfacesValidator
from ._streamtube import StreamtubesValidator
from ._splom import SplomsValidator
from ._scatterternary import ScatterternarysValidator
from ._scatter import ScattersValidator
from ._scatterpolar import ScatterpolarsValidator
from ._scatterpolargl import ScatterpolarglsValidator
from ._scattermapbox import ScattermapboxsValidator
from ._scattergl import ScatterglsValidator
from ._scattergeo import ScattergeosValidator
from ._scattercarpet import ScattercarpetsValidator
from ._scatter3d import Scatter3dsValidator
from ._sankey import SankeysValidator
from ._pointcloud import PointcloudsValidator
from ._pie import PiesValidator
from ._parcoords import ParcoordssValidator
from ._parcats import ParcatssValidator
from ._ohlc import OhlcsValidator
from ._mesh3d import Mesh3dsValidator
from ._histogram import HistogramsValidator
from ._histogram2d import Histogram2dsValidator
from ._histogram2dcontour import Histogram2dContoursValidator
from ._heatmap import HeatmapsValidator
from ._heatmapgl import HeatmapglsValidator
from ._contour import ContoursValidator
from ._contourcarpet import ContourcarpetsValidator
from ._cone import ConesValidator
from ._choropleth import ChoroplethsValidator
from ._carpet import CarpetsValidator
from ._candlestick import CandlesticksValidator
from ._box import BoxsValidator
from ._bar import BarsValidator
from ._barpolar import BarpolarsValidator
from ._area import AreasValidator
