from ._violin import ViolinValidator
from ._table import TableValidator
from ._surface import SurfaceValidator
from ._streamtube import StreamtubeValidator
from ._splom import SplomValidator
from ._scatterternary import ScatterternaryValidator
from ._scatterpolargl import ScatterpolarglValidator
from ._scatterpolar import ScatterpolarValidator
from ._scattermapbox import ScattermapboxValidator
from ._scattergl import ScatterglValidator
from ._scattergeo import ScattergeoValidator
from ._scattercarpet import ScattercarpetValidator
from ._scatter3d import Scatter3dValidator
from ._scatter import ScatterValidator
from ._sankey import SankeyValidator
from ._pointcloud import PointcloudValidator
from ._pie import PieValidator
from ._parcoords import ParcoordsValidator
from ._parcats import ParcatsValidator
from ._ohlc import OhlcValidator
from ._mesh3d import Mesh3dValidator
from ._histogram2dcontour import Histogram2dContourValidator
from ._histogram2d import Histogram2dValidator
from ._histogram import HistogramValidator
from ._heatmapgl import HeatmapglValidator
from ._heatmap import HeatmapValidator
from ._contourcarpet import ContourcarpetValidator
from ._contour import ContourValidator
from ._cone import ConeValidator
from ._choropleth import ChoroplethValidator
from ._carpet import CarpetValidator
from ._candlestick import CandlestickValidator
from ._box import BoxValidator
from ._barpolar import BarpolarValidator
from ._bar import BarValidator
from ._area import AreaValidator
from ._layout import LayoutValidator
from ._frames import FramesValidator
from ._data import DataValidator
