from ._valuesrc import ValuesrcValidator
from ._value import ValueValidator
from ._targetsrc import TargetsrcValidator
from ._target import TargetValidator
from ._sourcesrc import SourcesrcValidator
from ._source import SourceValidator
from ._line import LineValidator
from ._labelsrc import LabelsrcValidator
from ._label import LabelValidator
from ._hoverlabel import HoverlabelValidator
from ._hoverinfo import HoverinfoValidator
from ._colorsrc import ColorsrcValidator
from ._color import ColorValidator
